#set working directory
#setwd('..')

library(mvtnorm)
set.seed(1111111111)

##model related parameter
T		<- 	5
beta		<-	c(6,0.15)
lambda	<-	matrix(c(1,1,1,1,1,0,1,2,3,4),T,2)
mu		<-	lambda%*%beta


for(N in c(50,100,300,500,1000)){	##sample size

	for(sigE in c(1,3)){	##sigmaE = error variance

		for (sigLS in c(0,0.3)){	##sigmaLS = correlation(intercept,slope)
		
for(repl in 1:500){

	id 	<-	rep(repl,N) 
	covLS	<-	array(c(1,sigLS,sigLS,1), dim=c(2,2))

	e	<-	matrix(rnorm(N*T,0,sqrt(sigE)),N,T)
	u 	<-	rmvnorm(N,c(0,0),covLS) 
	b1	<-	u[,2]+beta[2]
	y	<-	matrix(rep(mu,N),N,T,byrow=TRUE)+t(lambda%*%t(u))+e

	##-----------------------complete data---------------------------------------
	##-----------------------------------------------------------------------
######## complete data:dat1
	yout	<-	cbind(id,y)
	name  <-	paste('dat1.N',N,'.sigE',sigE,'.sigLS',sigLS,'.txt',sep='') 
	write.table(yout,file=name,append=TRUE,row.names=F,col.names=F)


	##----------------------missing data-----------------------------------------
	##-----------------------------------------------------------------------
	for(mr in c(0.03,0.05,0.08,0.15)){	##missing rate = .03,.05,.08,.15

	
	##-----------------------MCAR data---------------------------------------

########MCAR: dat2##################################

	yMCAR1 <-	y	##yMCAR1:regular MCAR data
	for(j in 2:T){	
		for(i in 1:N){
			if(is.na(yMCAR1[i,j-1])){yMCAR1[i,j] <- NA}else{
			rn	<-	runif(1)	##random number
			if(rn<mr){yMCAR1[i,j] <- NA}}
		}
	}
	
	yMCAR1out <- cbind(id,yMCAR1)	

	name	<-	paste('dat2.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.txt',sep='')
	write.table(yMCAR1out,file=name,row.names=F,col.names=F,append=TRUE)
	
	##listwise deletion
	MCARdel <- na.omit(yMCAR1out)
	name	<-	paste('dat20.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.txt',sep='')
	write.table(MCARdel,file=name,row.names=F,col.names=F,append=TRUE)

########MCAR with refreshment: dat3##################################	
	sup <- length(which(is.na(yMCAR1[,2])))	##number of missingness at T2
	if(sup==0){sup <- round(mr*N)}

	for (coefN in 1:3){
	
		Nrf <- coefN*sup		##refreshment sample size
		Trf <- T-2	#the number of measurement occasions
		idrf 	<-	rep(repl,Nrf)
		erf	<-	matrix(rnorm(Nrf*Trf,0,sqrt(sigE)),Nrf,Trf)
		urf 	<-	rmvnorm(Nrf,c(0,0),covLS) 
		yrf	<-	matrix(rep(mu[3:5],Nrf),Nrf,Trf,byrow=TRUE)+t(lambda[3:5,]%*%t(urf))+erf

		yMCAR2	<-	yrf	##refreshment sample with MCAR
		if(length(c(yMCAR2))==T-2){yMCAR2 <- as.matrix(yMCAR2)}

		##missrate should be the same as MCAR1
		for(j in 2:Trf){	
			for(i in 1:Nrf){
				if(is.na(yMCAR2[i,j-1])){yMCAR2[i,j] <- NA}else{
				rn	<-	runif(1)
				if(rn<mr){yMCAR2[i,j] <- NA}}
			}
		}
	
		yMCAR2out <- cbind(idrf,matrix(NA,Nrf,2),yMCAR2)
		yMCAR2out <- rbind(yMCAR1out,yMCAR2out)
		name	<-	paste('dat3.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.coefN',coefN,'.txt',sep='')
		write.table(yMCAR2out,file=name,row.names=F,col.names=F,append=TRUE)

	}##coefN


	##-----------------------MAR data---------------------------------------

	##some calculation here: A=a*b1+ee,A:auxiliary variable, b1 and ee are independent.
	##let var(ee)=var(b1)=1, then corr(A,b1)=a/sqrt(a^2+1)
	##we want corr(A,b1) to be corAb=.3,.5,.8, correspondingly, a=corAb/sqrt(1-corAb^2)

for(corAb in c(.3,.5,.8)){

	a <- corAb/sqrt(1-corAb^2)
	Aux <- a*b1+rnorm(N,0,1)
	#Aux~N(a*beta[2],sqrt(a^2+1))

########MAR: dat4##################################

	yMAR1 <-	y	##yMAR1:regular MAR data

	for(j in 2:T){
		crit <- qnorm((1-mr)^(j-1),a*beta[2],sqrt(a^2+1))
		yMAR1[which(Aux>crit),j] <- NA
	}

	yMAR1out <- cbind(id,yMAR1,Aux)

	name	<-	paste('dat4.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.corAb',corAb,'.txt',sep='')
	write.table(yMAR1out,file=name,row.names=F,col.names=F,append=TRUE)

	##listwise deletion
	MARdel <- na.omit(yMAR1out)
	name	<-	paste('dat40.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.corAb',corAb,'.txt',sep='')
	write.table(MARdel,file=name,row.names=F,col.names=F,append=TRUE)


	##MAR with supplemental sample
	sup <- length(which(is.na(yMAR1[,2])))	##number of missingness at T2
	if(sup==0){sup <- round(mr*N)}

	for (coefN in 1:3){

		Nr <- coefN*sup		##supplemental sample size
		Tr <- T-2	#the number of measurement occasions
		idr 	<-	rep(repl,Nr)

########MAR with refreshment: dat5##################################		

		erf	<-	matrix(rnorm(Nr*T,0,sqrt(sigE)),Nr,T)
		urf 	<-	rmvnorm(Nr,c(0,0),covLS) 
		b1rf	<-	urf[,2]+beta[2]
		yrf	<-	matrix(rep(mu,Nr),Nr,T,byrow=TRUE)+t(lambda%*%t(urf))+erf
		
		Auxrf <- a*b1rf+rnorm(Nr,0,1)

		yMAR2 <- yrf[,3:T]	##refreshment sample with MAR
		if(length(c(yMAR2))==T-2){yMAR2 <- as.matrix(t(yMAR2))}	##change to matrix format when Nr=1

		for(j in 4:T){
			crit <- qnorm((1-mr)^(j-1),a*beta[2],sqrt(a^2+1))
			yMAR2[which(Auxrf>crit),(j-2)] <- NA
		}

		yMAR2out <- cbind(idr,matrix(NA,Nr,2),yMAR2,Auxrf)
		yMAR2out <- rbind(yMAR1out,yMAR2out)

		name	<-	paste('dat5.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.corAb',corAb,'.coefN',coefN,'.txt',sep='')
		write.table(yMAR2out,file=name,row.names=F,col.names=F,append=TRUE)

########MAR with replacement: dat6##################################

		yMAR3 <- NULL
		while(length(c(yMAR3))<Nr*(T+1)){
			erp	<-	matrix(rnorm(T,0,sqrt(sigE)),1,T)
			urp 	<-	rmvnorm(1,c(0,0),covLS) 
			b1rp	<-	urp[,2]+beta[2]
			yrp	<-	matrix(rep(mu,1),1,T,byrow=TRUE)+t(lambda%*%t(urp))+erp
			Auxrp <- a*b1rp+rnorm(1,0,1)
			if(Auxrp > a*beta[2]){yMAR3 <- rbind(yMAR3,c(yrp,Auxrp))} 			
		}

		Auxrp <- yMAR3[,(T+1)]
		yMAR3 <- yMAR3[,3:T]	##replacement sample with MAR
		if(length(c(yMAR3))==T-2){yMAR3 <- as.matrix(t(yMAR3))}	##change to matrix format when Nr=1

		for(j in 4:T){
			crit <- qnorm((1-mr)^(j-1),a*beta[2],sqrt(a^2+1))
			yMAR3[which(Auxrp>crit),(j-2)] <- NA
		}

		yMAR3out <- cbind(idr,matrix(NA,Nr,2),yMAR3,Auxrp)
		yMAR3out <- rbind(yMAR1out,yMAR3out)

		name	<-	paste('dat6.N',N,'.sigE',sigE,'.sigLS',sigLS,'.mr',mr,'.corAb',corAb,'.coefN',coefN,'.txt',sep='')
		write.table(yMAR3out,file=name,row.names=F,col.names=F,append=TRUE)



	}##coefN



}# corAb


}	#mr
}	#repl
}	#sigLS
}	#sigE
}	#N

